@@ -1,6 +1,6 @@
 from math import radians, cos, sin, asin, sqrt, degrees, pi, atan2
 from enum import Enum
-from typing import Union
+from typing import Union, Tuple
 
 
 # mean earth radius - https://en.wikipedia.org/wiki/Earth_radius#Mean_radius
@@ -60,7 +60,29 @@ def get_avg_earth_radius(unit):
     return _AVG_EARTH_RADIUS_KM * _CONVERSIONS[unit]
 
 
-def haversine(point1, point2, unit=Unit.KILOMETERS):
+def _normalize(lat: float, lon: float) -> Tuple[float, float]:
+    """
+    Normalize point to [-90, 90] latitude and [-180, 180] longitude.
+    """
+    lat = (lat + 90) % 360 - 90
+    if lat > 90:
+        lat = 180 - lat
+        lon += 180
+    lon = (lon + 180) % 360 - 180
+    return lat, lon
+
+
+def _ensure_lat_lon(lat: float, lon: float):
+    """
+    Ensure that the given latitude and longitude have proper values. An exception is raised if they are not.
+    """
+    if lat < -90 or lat > 90:
+        raise ValueError(f"Latitude {lat} is out of range [-90, 90]")
+    if lon < -180 or lon > 180:
+        raise ValueError(f"Longitude {lon} is out of range [-180, 180]")
+
+
+def haversine(point1, point2, unit=Unit.KILOMETERS, normalize=False):
     """ Calculate the great-circle distance between two points on the Earth surface.
 
     Takes two 2-tuples, containing the latitude and longitude of each point in decimal degrees,
@@ -71,6 +93,7 @@ def haversine(point1, point2, unit=Unit.KILOMETERS):
     :param unit: a member of haversine.Unit, or, equivalently, a string containing the
                  initials of its corresponding unit of measurement (i.e. miles = mi)
                  default 'km' (kilometers).
+    :param normalize: if True, normalize the points to [-90, 90] latitude and [-180, 180] longitude.
 
     Example: ``haversine((45.7597, 4.8422), (48.8567, 2.3508), unit=Unit.METERS)``
 
@@ -88,6 +111,14 @@ def haversine(point1, point2, unit=Unit.KILOMETERS):
     lat1, lng1 = point1
     lat2, lng2 = point2
 
+    # normalize points or ensure they are proper lat/lon, i.e., in [-90, 90] and [-180, 180]
+    if normalize:
+        lat1, lng1 = _normalize(lat1, lng1)
+        lat2, lng2 = _normalize(lat2, lng2)
+    else:
+        _ensure_lat_lon(lat1, lng1)
+        _ensure_lat_lon(lat2, lng2)
+
     # convert all latitudes/longitudes from decimal degrees to radians
     lat1 = radians(lat1)
     lng1 = radians(lng1)
@@ -102,7 +133,7 @@ def haversine(point1, point2, unit=Unit.KILOMETERS):
     return 2 * get_avg_earth_radius(unit) * asin(sqrt(d))
 
 
-def haversine_vector(array1, array2, unit=Unit.KILOMETERS, comb=False):
+def haversine_vector(array1, array2, unit=Unit.KILOMETERS, comb=False, normalize=False):
     '''
     The exact same function as "haversine", except that this
     version replaces math functions with numpy functions.
@@ -133,6 +164,14 @@ def haversine_vector(array1, array2, unit=Unit.KILOMETERS, comb=False):
         if array1.shape != array2.shape:
             raise IndexError("When not in combination mode, arrays must be of same size. If mode is required, use comb=True as argument.")
 
+    # normalize points or ensure they are proper lat/lon, i.e., in [-90, 90] and [-180, 180]
+    if normalize:
+        array1 = numpy.array([_normalize(p[0], p[1]) for p in array1])
+        array2 = numpy.array([_normalize(p[0], p[1]) for p in array2])
+    else:
+        [_ensure_lat_lon(p[0], p[1]) for p in array1]
+        [_ensure_lat_lon(p[0], p[1]) for p in array2]
+
     # unpack latitude/longitude
     lat1, lng1 = array1[:, 0], array1[:, 1]
     lat2, lng2 = array2[:, 0], array2[:, 1]
@@ -159,7 +198,7 @@ def haversine_vector(array1, array2, unit=Unit.KILOMETERS, comb=False):
     return 2 * get_avg_earth_radius(unit) * numpy.arcsin(numpy.sqrt(d))
 
 
-def inverse_haversine(point, distance, direction, unit=Unit.KILOMETERS):
+def inverse_haversine(point, distance, direction: Union[Direction, float], unit=Unit.KILOMETERS):
 
     lat, lng = point
     lat, lng = map(radians, (lat, lng))
